/****************************************************************************
**
** Copyright (C) Datafox GmbH.
** All rights reserved.
** Contact: Datafox GmbH (develop@datafox.de)
**
** This file is part of the DFComDLL module of the Datafox SDK.
**
****************************************************************************/
#pragma once
#ifndef DATAFOX_COMMUNICATION_MODULE_BASE_H
#define DATAFOX_COMMUNICATION_MODULE_BASE_H

// Protokollklasse einbinden.
#include "datafox/communication/protocol.h"

#include <boost/property_tree/ptree.hpp>
#include <boost/property_tree/json_parser.hpp>

#ifdef WITH_VIRTUAL_DEVICE
#include <boost/uuid/uuid.hpp>
#include <boost/uuid/uuid_generators.hpp>
#include <boost/uuid/uuid_io.hpp>

#include <boost/filesystem.hpp>
#include <string>
#endif

/** In diesem Module sind alle Basisspezifischen Modulfunktionen enthalten.
*
* 
*
*/

namespace Datafox
{
    namespace Communication
    {
        // Fuer die Verwendung der Verbindung im Active-Mode
        enum ActiveModeState
        { 
            ams_unused = 0, // Verbindungsklasse ist nicht in Verwendung
            ams_starting,   // Verbindungsklasse wird vorbereitet.
            ams_accepting,  // Verbindungsklasse nimmt eine Verbindung entgegen.
            ams_detecting,  // Verbindungsklasse ermittelt Daten vom Geraet.
            ams_usable,     // Verbindungsklasse hat die zu ermittelten Daten vom Geraet erhalten.
            ams_binding,    // Verbindungsklasse durchläuft die Bindungsprüfung.
            ams_active      // Verbindungsklasse befindet sich im Active-Mode.
        };

        class ModuleBase
        {
        public:
            ModuleBase( int32_t connectionNumber = ConnectionBase::defaultConnectionNumber(), uint8_t deviceAddress = ProtocolBase::defaultDeviceAddress() );
            virtual ~ModuleBase();

        private:
            std::shared_ptr<Protocol> m_protocol;

        public:
            std::shared_ptr<Protocol> protocolPointer()
            {
                return m_protocol;
            }
            void setProtocolPointer( std::shared_ptr<Protocol> base )
            {
                m_protocol = base;
            }

            Protocol& protocol()
            {
                return *m_protocol.get();
            }
        private:
            std::shared_ptr<TraceBase> m_trace;

        public:
            std::shared_ptr<TraceBase> tracePointer()
            {
                return m_trace;
            }
            void setTracePointer( std::shared_ptr<TraceBase> base )
            {
                m_trace = base;
            }

        public:
            ConnectionBase& connection()
            {
                return protocol().connection();
            }

            /// Oeffnen der Schnittstelle
            int openConnection( ConnectionBase::ConnectionType type, const std::string& connectionString, uint32_t connectionValue, uint32_t communicationReadTimeout )
            {
                return connection().open( type, connectionString, connectionValue, communicationReadTimeout );
            }
            /// Schliessen der Schnittstelle.
            virtual int closeConnection()
            {
                protocol().aesCryptSession().resetSessionKey();
                return connection().close();
            }

            bool isConnectionOpen()
            {
                return connection().isOpen();
            }

        public:
            virtual void trace( const char *format... )
            {
                if ( m_trace.get() != nullptr && m_trace->mode() >= TraceBase::TraceMode::on)
                {
                    va_list args;
                    va_start( args, format );
                    m_trace->print( format, args );
                    va_end( args );
                }
            }

            void setTraceMode( TraceBase::TraceMode mode = TraceBase::TraceMode::on )
            {
                if ( m_trace.get() != nullptr )
                {
                    m_trace->setMode( mode );
                }
            }
            TraceBase::TraceMode traceMode()
            {
                TraceBase::TraceMode mode = TraceBase::TraceMode::off;
                if ( m_trace.get() != nullptr )
                {
                    mode = m_trace->mode();
                }

                return mode;
            }

            void setTraceFilePath( const std::string &file_path )
            {
                if ( m_trace.get() != nullptr )
                {
                    m_trace->setFilePath( file_path );
                }
            }
            std::string traceFilePath()
            {
                std::string file_path;
                if ( m_trace.get() != nullptr )
                {
                    file_path = m_trace->filePath();
                }

                return file_path;
            }

        protected:
            bool m_autoBaud;

        public:
            // Autobauding mit prio 38400, 9600, 19200. ( HK hinzugefügt 57600, 115200 )
            bool autoBaud()
            {
                return m_autoBaud;
            }
            void setAutoBaud( bool val )
            {
                m_autoBaud = val;
            }

            void reset();

            // Fehlerbearbeitung
        public:
            void resetError();

            // Fehlertexte fuer die Module/Funktionen
            uint16_t m_errorModuleNumber;
            std::string m_errorModuleText;
            virtual void errorModule( uint16_t module, uint16_t error, std::string &text );

            uint16_t m_errorFunctionNumber;
            std::string m_errorFunctionText;
            virtual void errorFunction( uint16_t module, uint16_t func, uint16_t error, std::string &text );

            void traceError( uint8_t device_address, uint16_t module, uint16_t func, int32_t error );

            uint8_t m_packetData[IV_PACKET_LENGTH_DATA]; // 21 Byte Paketframe

        private:
            volatile ActiveModeState m_activeModeState;

        public:
            // Verbindungsobjekt im Active-Mode Verwendet.
            inline void setActiveModeState( ActiveModeState state ) 
            { 
                m_activeModeState = state; 
            }
            inline ActiveModeState activeModeState() 
            { 
                return m_activeModeState; 
            }

        protected:
            std::recursive_mutex m_mutexCall; // Eine in sich geschlossene Kommunikation Paketaustausch. Inkl. Aktive-Antwort
            typedef std::lock_guard<std::recursive_mutex> lockGuardCall;

        protected:
            // Call-Back-Funktion
            std::function<int32_t (uint32_t size, uint32_t count) > m_callBackFunctionWithParameters;
            std::function<int32_t( int32_t connectionNumber, int32_t deviceAddress, uint32_t blockCount, uint32_t blockNumber, uint32_t sizeBlock, uint32_t sizeTransfered ) > m_callBackBlockTransfer;
            uint32_t m_blockTransferCount;
            uint32_t m_blockTransferNumber;
        public:
            void setCallBackFunctionWithParameters( std::function<int32_t ( uint32_t, uint32_t )> func );
            void setCallBackBlockTransfer( std::function<int32_t( int32_t, int32_t, uint32_t, uint32_t, uint32_t, uint32_t )> func );

            //#################################
            // CallRoutinen
        public:
            // Ausfuehren eines Protokollbefehls
            virtual int32_t call( uint16_t context, uint16_t module, uint16_t func, uint8_t *buffer, uint16_t &count, int32_t contextValue = -1 );
            // Ausfuehren von Befehlen
            virtual int32_t remoteCall( uint16_t module, uint16_t func, uint8_t *data, uint16_t &count );

        protected:
            int32_t m_result;         // Returnwert der call-Routine

        public:
            inline int32_t lastResult()
            { 
                return m_result;
            }

            void setConnectionNumber( int32_t number )
            {
                connection().setNumber( number );
            }
            int32_t connectionNumber()
            {
                return connection().number();
            }

            bool isOwnDeviceNumber( uint8_t deviceAddress )
            {
                return protocol().isOwnDeviceNumber( deviceAddress );
            }

            void setDeviceNumberReceiver( uint8_t deviceAddress )
            {
                protocol().setDeviceNumberReceiver( deviceAddress );
            }
            uint8_t deviceNumberReceiver()
            {
                return protocol().deviceNumberReceiver();
            }

            void setDeviceNumberSender( uint8_t deviceAddress )
            {
                protocol().setDeviceNumberSender( deviceAddress );
            }
            uint8_t deviceNumberSender()
            {
                return protocol().deviceNumberSender();
            }

            uint32_t crc32( uint8_t *data, std::size_t size, uint32_t crc = 0xFFFFFFFF, bool closure = true )
            {
                return protocol().crc32( data, size, crc, closure );
            }

        public:
            // Einfache Anfrage ob das Geraet antwortet.
            int32_t ping( uint8_t device_address, uint8_t *data, uint16_t *count );

        public:
            template<class Type>
            Type property( const std::string &name, Type defaultValue )
            {
                return m_deviceProperties.get<Type>( name, defaultValue );
            }

            template<class Type>
            void setProperty( const std::string &name, Type value )
            {
                m_deviceProperties.put<Type>( name, value );
            }

        private:
            boost::property_tree::ptree m_deviceProperties;

#ifdef WITH_VIRTUAL_DEVICE
        public:
            void setUuid( boost::uuids::uuid &uuid );
            boost::uuids::uuid uuid();
            std::string uuidAsString();
    
            void setPathCache( const boost::filesystem::path &path );
            boost::filesystem::path pathCache();
    
            std::string pathCacheDevice();
            std::string filePathDevice();

        protected:
            virtual void loadFromCache();
            virtual void saveToCache();

        private:
            boost::filesystem::path m_pathCache;
            boost::uuids::uuid m_uuid;
#endif
        };

    } // namespace Communication
} // namespace Datafox

#endif
