/****************************************************************************
**
** Copyright (C) Datafox GmbH.
** All rights reserved.
** Contact: Datafox GmbH (develop@datafox.de)
**
** This file is part of the DFComDLL module of the Datafox SDK.
**
****************************************************************************/
#pragma once
#ifndef DATAFOX_COMMUNICATION_MODULE_SYSTEM_H
#define DATAFOX_COMMUNICATION_MODULE_SYSTEM_H

// Basisklasse einbinden
#include "datafox/communication/module_hardware.h"
// Verwaltungsklasse fuer BlockTransfer
#include "datafox/communication/controller_segment.h"

#include <boost/ptr_container/ptr_map.hpp>
#include <set>
//#include <sstream>
//#include <string>

#ifdef WITH_VIRTUAL_DEVICE
#include <boost/filesystem.hpp>
#include <fstream>
#include <memory>

#include <boost/format.hpp>
#include <map>
typedef std::map<std::string, std::string> type_variable;
#endif

/** In diesem Module sind alle Systemspezifischen Funktionen enthalten.
 *
 */
#define IV_MODULE_SYSTEM                         0x0001


// Ermitteln der Firmwareversion
#define IV_FUNCTION_GET_VERSION                  0x0000

// Setzen/Ermitteln der Seriennummer
#define IV_FUNCTION_SET_SERIAL                   0x0001
#define IV_FUNCTION_GET_SERIAL                   0x0002
    
// Setzen/Ermitteln der Systemzeit. Format (YYYY-MM-DD HH:MM:SS\0)
#define IV_FUNCTION_SET_RTC                      0x0003
#define IV_FUNCTION_GET_RTC                      0x0004


// Setzen/Ermitteln einer Systemvariablen 
#define IV_FUNCTION_SET_SYSTEM_VARIABLE          0x0005
#define IV_FUNCTION_GET_SYSTEM_VARIABLE          0x0006

// Der Name einer Systemvariablen setzt sich auf folgende Art zusammen:
// MAJOR_NAME.MINOR_NAME
// Für GPRS ist der Majorname MODEM_MC35i.
// Fehlernummer:
// 1 = MAJOR_NAME ist unbekannt
// 2 = MINOR_NAME ist unbekannt
// 3 = Länge des Variablenwertes ist zu lang.


// Setzen/Ermitteln eines Relaiszustandes
#define IV_FUNCTION_SET_RELAY                    0x0007
#define IV_FUNCTION_GET_RELAY                    0x0008

// Die Namen sind: INTERN, TMR (Türmodule), IOM (IOModule)
// Die Busnummer ist die Busnummer bei 0 beginnend. (Bei INTERN eigentlich immer 0)
// Die Relaisnummer ist die Relaisnummer bei 0 beginnend.
// Die Zeit ist eine Angabe in Sekunden (0 bedeutet eine Dauerschaltung in den gewählten Status)
// Fehlernummer:
// 1 = Name undefiniert
// 2 = Busnummer undefiniert
// 3 = Relaisnummer undefiniert
// 4 = Status undefiniert  (0, 1) (Könnte irgendwann um andere Varianten erweitert werden)

// Schreiben/Lesen von: Externen Flash
#define IV_FUNCTION_SET_FLASH                    0x0009
#define IV_FUNCTION_GET_FLASH                    0x000A

// Schreiben/Lesen von: EEPROM
#define IV_FUNCTION_SET_EEPROM                   0x000B
#define IV_FUNCTION_GET_EEPROM                   0x000C

// Schreiben/Lesen von: Program-Flash schreiben/lesen
#define IV_FUNCTION_SET_PROGRAM                  0x000D // (wird nur aufgenommen)
#define IV_FUNCTION_GET_PROGRAM                  0x000E

// Starten/Beenden von: Segmentuebertragung (inklusive aktiver Antwort)
#define IV_FUNCTION_BEGIN_SET_SEGMENT                0x000F
#define IV_FUNCTION_BEGIN_SET_SEGMENT_OK             0x0010
#define IV_FUNCTION_END_SET_SEGMENT                  0x0011
// Im Bestaetigungspaket fuer Befehlerhalt ist der CRC = 0 und 
// im Bestaetigungspaket fuer CRC pruefung ist der CRC = dem bei SYSTEM_START gesendeten.
// Ist der CRC falsch, gibt es ein Fehlerpaket mit Fehlernummer 6.

#ifdef WITH_VIRTUAL_DEVICE
enum class Segment_type
{
    Update,
    Update_background,
    Firmware,
    Fonts,
    Grafics,
    Setup,
    List,
    Setup_extended,
    List_access,
    List_timeboy,
    Fingerprint,
    Text,
    Toch_keys,
    Display_colors,
    Wlan_configs,
    Hardware_configs,
    File_transfer
};

struct Segment_parameter
{
    Segment_type type;
    uint32_t      address;
    uint32_t      size;
    uint32_t      crc;
};

struct Segment_data
{
    Segment_parameter parameter;
    boost::filesystem::path file_path;
    std::shared_ptr<std::fstream> file_stream;
};
#endif

// Die folgenden IV_VALUE_SEGMENT_.. gelten fuer _BEGIN_SEGMENT und _END_SEGMENT
#define IV_VALUE_SEGMENT_UPDATE                            0 // Es werden Daten gesetzt die nachdem Uebernommen werden.
#define IV_VALUE_SEGMENT_UPDATE_BG                         1 // Es werden Daten gesetzt die nachdem im Hintergrund uebernommen werden.
#define IV_VALUE_SEGMENT_FIRMWARE                          2
#define IV_VALUE_SEGMENT_FONTS                             3
#define IV_VALUE_SEGMENT_GRAFICS                           4
#define IV_VALUE_SEGMENT_SETUP                             5
#define IV_VALUE_SEGMENT_LIST                              6
#define IV_VALUE_SEGMENT_SETUP_EXTENDED                    7
#define IV_VALUE_SEGMENT_LIST_ACCESS                       8 // Listen fuer ZK II werden uebertragen, dadurch wird die Zutrittskontrolle blockiert.
#define IV_VALUE_SEGMENT_LIST_TIMEBOY                      9 // Liste fuer Timeboy uebertragen
#define IV_VALUE_SEGMENT_FINGERPRINT                       10
#define IV_VALUE_SEGMENT_TEXT                              11
#define IV_VALUE_SEGMENT_TOUCHKEYS                         12
#define IV_VALUE_SEGMENT_DISPLAYCOLORS                     13
#define IV_VALUE_SEGMENT_WLANCONFIGS                       14
#define IV_VALUE_SEGMENT_HARDWARECONFIGS                   15
#define IV_VALUE_SEGMENT_FILETRANSFER                      16

// 0  -> Global fuer Uebernahmerahmen von Datenbundel mit Bedienstopp. Z. B. Firmware, Setup
// 1  -> Global fuer Uebernahmerahmen von Datenbundel ohne Bedienstopp. Z. B. Listen
// 2  -> Firmware
// 3  -> Fonts
// 4  -> Grafik
// 5  -> Setup
// 6  -> Listen fuer Setup.
// 7  -> Setup Extended Bereich.
// 8  -> Listen fuer Zutrittskontrolle.
// 9  -> Listen fuer Timeboys.
// 10 -> Firmware des Fingerprintmoduls uebertragen
// 11 -> Segment fuer Textdaten der Firmware uebertragen
// 12 -> Segment fuer Daten der Tasten auf Touch übertragen
// 13 -> Segment fuer die Farbdaten des Displays
// 14 -> Segment fuer die WLAN Konfigurationen des Redpine Moduls
// 15 -> Segment für die Hardwarekonfiguration
// 16 -> Segment mit Dateidaten.
// Fehler:
// 1 -> Unbekannter SS-Type
// 2 -> Zu viele Updatesegmente (7 vordefiniert)
// 3 -> Angeforderter Speicherplatz steht nicht zur Verfügung
// 4 -> Kein Stopp vor Start
// 5 -> Verschachtelungsfehler von Stopp und Start. {[]()} -> OK, aber {[( -> Fehler
// 6 -> CRC Berechnung schlug fehl.

#define IV_FUNCTION_END_SET_SEGMENT_OK               0x0012
// Wird vom Geraet aufgrund eines SYSTEM_START -> SYSTEM_END mit nachfolgender Berechnung vom CRC gesendet.

// Ermitteln von: Bootloaderversion
#define IV_FUNCTION_GET_VERSION_BOOTLOADER       0x0013
// Format: XX.XX.XX??????????????

// Durchfuehren eines Resets der Speicherbereiche
#define IV_FUNCTION_SYSTEM_RESET                 0x0014

// Resetmodus:
#define IV_VALUE_RESET_ALL             0 // Geraet in einen Zustand versetzen, wie nach dem Bootloader einspielen.
#define IV_VALUE_RESET_RESTART         1 // Warmstart (Watchdog ausführen)
#define IV_VALUE_RESET_FORMAT          2 // Flash formatieren.
#define IV_VALUE_RESET_SETUP           3 // Setup, Listen und Datensatzzeiger Löschen/Zurücksetzen

// Fehler:
// 1 -> Resetmodus unbekannt.


// Ins Bios wechseln
#define IV_FUNCTION_ENTER_BIOS			         0x0015
// Bioswechsel erfolgt
#define IV_FUNCTION_ENTERED_BIOS		         0x0016

// Bios verlassen (aenderungen uebernehmen
#define IV_FUNCTION_EXIT_BIOS			         0x0017
// Biosverlassen erfolgt
#define IV_FUNCTION_EXITED_BIOS			         0x0018

// Setzt/liest das zu verwendende Interface
#define IV_FUNCTION_SET_INTERFACE                0x0019
// 0 -> RS232
// 1 -> TCP/IP
// ...
#define IV_FUNCTION_GET_INTERFACE                0x001A

#define IV_VALUE_INTERFACE_RS232       0
#define IV_VALUE_INTERFACE_TCPIP       1 // Auch WLAN Matchport
#define IV_VALUE_INTERFACE_GSM         2
#define IV_VALUE_INTERFACE_MOBILE      3 // GPRS, LTE, etc..
#define IV_VALUE_INTERFACE_RS485       4
#define IV_VALUE_INTERFACE_MMC         5
#define IV_VALUE_INTERFACE_USB         6
#define IV_VALUE_INTERFACE_BLUETOOTH   7
#define IV_VALUE_INTERFACE_USB_HOST    9
#define IV_VALUE_INTERFACE_WLAN       10 // Redpine

// Setzen/Ermitteln der zu verwendende BaudRate
#define IV_FUNCTION_SET_BAUD                     0x001B
#define IV_FUNCTION_GET_BAUD                     0x001C
// Setzen/Ermitteln der zu verwendenden DeviceID
#define IV_FUNCTION_SET_DEVICE_ID                0x001D
#define IV_FUNCTION_GET_DEVICE_ID                0x001E

// Befehl der beim Geraetestart den Bootloader des MasterIV wiederherstellt.
#define IV_FUNCTION_ENTER_BOOTLOADER             0x001F

// In Monitor Mode von XPort wechselt -> siehe auch #define IV_FUNCTION_ENTER_XPORT_MONITOR_EXT		 0x0028 
#define IV_FUNCTION_ENTER_XPORT_MONITOR          0x0020
// Antwort in Monitor Mode gewechselt -> siehe auch #define IV_FUNCTION_ENTER_XPORT_MONITOR_EXT_OK   0x0029
#define IV_FUNCTION_ENTER_XPORT_MONITOR_OK       0x0021

// Monitorbefehl an XPort und Antwort erhalten.
#define IV_FUNCTION_CMD_XPORT                    0x0022

// In Monitor Mode von XPort wechselt -> siehe auch #define 
#define IV_FUNCTION_EXIT_XPORT_MONITOR           0x0023
// Antwort in Monitor Mode gewechselt -> siehe auch #define 
#define IV_FUNCTION_EXIT_XPORT_MONITOR_OK        0x0024


#define IV_FUNCTION_BEGIN_GET_SEGMENT            0x0025
// [in]-> [UINT16(SS_...)]
// <- 16bit 
#define IV_FUNCTION_BEGIN_GET_SEGMENT_OK         0x0026
// [out]-> 16bit Typ des Speichers
// [out]-> 32bit Adresse fuer Lesen
// [out]-> 32bit Groesse des Blocks
// [out]-> 32bit CRC des Blocks

#define IV_FUNCTION_END_GET_SEGMENT              0x0027
// -> [UINT16(SS_...)]
// <- [UINT16(SS_...)]
// Im Bestaetigungspaket fuer Befehlerhalt ist der CRC = 0 und 
// im Bestaetigungspaket fuer CRC pruefung ist der CRC = dem bei SYSTEM_START gesendeten.
// Ist der CRC falsch, gibt es ein Fehlerpaket mit Fehlernummer 6.

// Die folgenden IV_VALUE_SEGMENT_.. gelten fuer _BEGIN_GET_SEGMENT und _END_GET_SEGMENT
#define IV_VALUE_SEGMENT_GET_UPDATE         0 // Es werden Daten gelesen ohne Bedienstop.
#define IV_VALUE_SEGMENT_GET_UPDATE_BG      1 // Es werden Daten gesetzt die nachdem im Hintergrund uebernommen werden.
#define IV_VALUE_SEGMENT_GET_SETUP          5
#define IV_VALUE_SEGMENT_GET_SETUP_EXTENDED 7

// Fehler:
// 1 -> Unbekannter SS-Type
// 2 -> Zu viele Updatesegmente (7 vordefiniert)
// 3 -> Angeforderter Speicherplatz steht nicht zur Verfügung
// 4 -> Kein Stopp vor Start
// 5 -> Verschachtelungsfehler von Stopp und Start. {[]()} -> OK, aber {[( -> Fehler
// 6 -> CRC Berechnung schlug fehl.

#define IV_FUNCTION_END_GET_SEGMENT_OK           0x0028
// Wird vom Geraet aufgrund eines SYSTEM_START -> SYSTEM_END mit nachfolgender Berechnung vom CRC gesendet.

#define IV_FUNCTION_SYSTEM_RESET_OK              0x0029

// Transparentmodus aktivieren/deaktivieren.
#define IV_FUNCTION_OPEN_COM_SERVER_MODE         0x002A
#define IV_FUNCTION_CLOSE_COM_SERVER_MODE        0x002B

// Abfrage von Information zu einem Modultyp
#define IV_FUNCTION_GET_INFO                     0x002C
// param1: ACCESS.MODULE
// param2: bei ACCESS.MODULE UCHAR(BusNr) UCHAR(0=TM,1=L)

// Antwort in Com-Server-Mode gewechselt
#define IV_FUNCTION_OPEN_COM_SERVER_MODE_OK      0x002D

// Aktivieren/Deaktivieren und Abfragen der Benachrichtigungen
#define IV_FUNCTION_SET_NOTIFY                   0x002E
// param1: Typ der Benachrichtigung
// param2: Zu setztender Zustand der Benachrichtigung
#define IV_FUNCTION_GET_NOTIFY                   0x002F
// param1: Typ der Benachrichtigung
// Werte siehe IV_MODULE_APPLICATION.IV_FUNCTION_NOTIFY

#define IV_FUNCTION_GET_SOFTWARE_INFO            0x0030

// Lesen einer Seite des Flash (Bereich: 0 - 4095, Seite hat 528Byte)
#define IV_FUNCTION_GET_FLASH_PAGE               0x0031

// Setzen/Lesen der Systemflags von EEPROM-Adresse: 0x0ECC
// Bit1: Vorhandenes Setup und Listen werden übernommen und durch das Update nicht gelöscht.
//       Nur Mobil-MasterIV. Die Mindestversion auf dem Gerät muß 04.01.04.32 sein und die aufzuspielende mindestens 04.01.04.59 damit diese Option korrekt unterstützt wird.;
// Bit2-32: Reserved
#define IV_FUNCTION_SET_SYSTEM_FLAGS             0x0032
#define IV_FUNCTION_GET_SYSTEM_FLAGS             0x0033

#define IV_VALUE_SYSTEM_FLAG_MASK                0x0000FFFF
#define IV_VALUE_SYSTEM_FLAG_DEL_SETUP_AND_LISTS 0x00000001


// Schreiben/Lesen von: RAM
#define IV_FUNCTION_SET_RAM                      0x0034
#define IV_FUNCTION_GET_RAM                      0x0035

// Schreiben/Lesen von MMC
#define IV_FUNCTION_SET_MMC                      0x0036
#define IV_FUNCTION_GET_MMC                      0x0037
#define IV_FUNCTION_GET_SFSMPB                   0x0038

#pragma  pack(1)
// Strucktur fuer dynamische Verwaltung des SFS (MS) 06.11.08
typedef struct struct_memory_partition_block
{
    uint32_t  StorageStartAddress; ///< Startadresse des Filessystems im Speicher
    uint32_t  StartFATAddress;     ///< Startadresse FAT
    uint32_t  DirStartAddress;     ///< Startadresse File Directory
    uint32_t  FileStartAddress;    ///< Startadresse Datenbereich
    uint32_t  FirstRecordAddress;  ///< Startadresse des Ringspeichers
    uint32_t  StorageEndAddress;   ///< Letzte Adresse des Speicherbereiches

    uint16_t  DirStartCluster;     ///< Start Cluster in dem das Dateiverzeichnis liegt
    uint16_t  FileStartCluster;    ///< Start Cluster für die Daten der Dateien
    uint16_t  FATClusterCount;     ///< Anzahl der Cluster die von der FAT belegt werden
    uint16_t  DirClusterCount;     ///< Anzahl der Cluster die vom Verzeichnis belegt werden.
    uint16_t  FileClusterCount;    ///< Anzahl der Cluster für den Dateibereich
    uint16_t  TotalClusterCount;   ///< Anzahl aller Cluster des Dateisystems
    uint16_t  SectorsPerCluster;   ///< Anzahl der Sektoren pro Cluster

    uint32_t  ClusterSize;         ///< Größe eines Clusters in Byte
    uint32_t  RecordVolumeSize;    ///< Größe des Ringspeichers in Byte
    uint32_t  ListVolumeSize;      ///< Größe des Dateisystem in Byte
    uint32_t  StorageVolumeSize;   ///< Größe der Memory-Card in Byte

    uint8_t ClusterShiftBit;     ///< Schiebebit für Clusteradressierung

} SFSMPB;
#pragma pack()

// MatchPort in den Monitor Mode (z-Mode mit aktivem WLAN Interface) wechseln
#define IV_FUNCTION_ENTER_XPORT_MONITOR_EXT		 0x0039
// Antwort auf den Wechsel-Befehl BetriebsMode->MonitorMode
#define IV_FUNCTION_ENTER_XPORT_MONITOR_EXT_OK   0x003A

// System Device Log
#define IV_FUNCTION_GET_SYS_RECORD				 0x003B
#define IV_FUNCTION_SET_SYS_RECORD				 0x003C
#define IV_FUNCTION_RESTORE_ALLSYS_RECORDS		 0x003D

#define IV_ANSW_NO_SYS_RECORD               0xC0
#define IV_ANSW_ONE_SYS_RECORD				0xC1
#define IV_ANSW_ACK_ON_SYS_RECORD			0xC8
#define IV_SYS_RECORDS_MAX					0x200

// Passwort fuer Kommunikation
#define IV_FUNCTION_PASSWORD_CHECK              0x003E
#define IV_FUNCTION_PASSWORD_KEY                0x003F

#define IV_FUNCTION_PRESS_VIRTUAL_KEY           0x0040

// Schreiben eines Datenstroms
#define IV_FUNCTION_TRANSMIT_STREAM             0x0041
#define IV_FUNCTION_TRANSMIT_STREAM_OK          0x0042

#define IV_FUNCTION_RECEIVE_STREAM              0x0043

enum class Stream_type
{
    Update = 0,
    Update_background,
    Firmware,
    Fonts,
    Graphics,
    Setup,
    Setup_list,
    Setup_extended,
    List_access,
    List_timeboy,
    Fingerprint,
    Text,
    Touch_keys,
    Display_colors,
    Wlan_configs,
    Fake
};

// Passwort fuer Systemkonfigurationen
#define IV_FUNCTION_SYSTEM_KEY_CHECK          0x0044
#define IV_FUNCTION_SYSTEM_KEY                0x0045

// Lesen der Flashdaten über die physikalische Adresse
#define IV_FUNCTION_GET_FLASH_PHYSICAL        0x0046

// Befehl zum Transparenten durchsenden von Daten.
// Hierbei werden die MasterIV-spezifischen Zeichen wie '~' maskiert gesendet,
// der Empfänger demaskiert diese je nach Param2-Flag, ebenso bei dem Empfang.
#define IV_FUNCTION_TRANSPARENCY_MODE         0x0047
// Param1: uint16_t Slot-Nummer oder id zu der durchgeschaltet werden soll.
// 1 - 254 -> Slotnummer
// 255 - .. -> Andere Bedeutung. Daten zur Id aus der bios_settings.
// Param2: uint16_t Flags
// Bit1: Wenn gesetzt (Raw-Modus) werden alle Zeichen 1:1 durchgeleitet. Ansonsten (Masked-Modus) werden die empfangenen bzw. zu sendenden Zeichen de-/maskiert.
// Bit2: Wenn gesetzt wird die Zeitangabe in Param5 in Sekunden gewertet, ansonsten in Millisekunden.
// Bit3: Wenn gesetzt sollen nur Timeboy I-III Geräte angesprochen werden. Da TimeboyIV Geräte einen TimeboyIII in ihrer Kommunikation nachahmen können
//       wird dieses Flag dazu verwendet, Bereits beim Herstellen des TransparentMode ältere Timeboystände zu verlangen. Nutzung im TbSetup und TbCom.dll
// Bit4-16: Unbenutzt
// Param3: uint32_t gewünschte Lesepuffergröße der Gegenstelle in Byte.
// Param4: uint32_t gewünschte Schreibpuffergröße der Gegenstelle in Byte.
// Param5: uint16_t Timeout bei Kommunikation über Raw-Modus in Milisekunden oder Sekunden, je nach Flag-Bit2.

// Befehle zum Schalten der gelben LED bei Dockingstation-V2
#define IV_FUNCTION_SET_LED                   0x0048
#define IV_FUNCTION_GET_LED                   0x0049
// Param1: uint8_t slot. Welcher Slot angesprochen werden soll.
// 0 -> Das Gerät mit der deviceId selbst.
// 1 - 253 -> Slotnummer
// Param2: uint8_t Led-Id welche LED angesprochen wird.
// Bei der Mehrfachdocking entspricht diese Id folgenden LED's.
// 1 -> Rote
// 2 -> Gelbe
// 3 -> Grüne
// Param3: uint8_t Status der Blinkfrequenz.
// Hierbei handelt es sich um die Prozentangabe der Zeit relativ zu einer Sekunde.
// 6,66% sind 66,6ms an, 933,4ms aus.
// 93,33% sind 933,3ms an, 66,7ms aus.
#define IV_VALUE_LED_OFF             ( 0x00 )    // Flag LedOff
#define IV_VALUE_LED_PERCENT01       ( 0x01 )    // Flag LedOn6,66%
#define IV_VALUE_LED_PERCENT02       ( 0x02 )    // Flag LedOn13,33%
#define IV_VALUE_LED_PERCENT03       ( 0x03 )    // Flag LedOn19,99%
#define IV_VALUE_LED_PERCENT04       ( 0x04 )    // Flag LedOn26,66%
#define IV_VALUE_LED_PERCENT05       ( 0x05 )    // Flag LedOn33,33%
#define IV_VALUE_LED_PERCENT06       ( 0x06 )    // Flag LedOn39,99%
#define IV_VALUE_LED_PERCENT07       ( 0x07 )    // Flag LedOn46,66%
#define IV_VALUE_LED_PERCENT08       ( 0x08 )    // Flag LedOn53,33%
#define IV_VALUE_LED_PERCENT09       ( 0x09 )    // Flag LedOn59,99%
#define IV_VALUE_LED_PERCENT10       ( 0x0A )    // Flag LedOn66,66%
#define IV_VALUE_LED_PERCENT11       ( 0x0B )    // Flag LedOn73,33%
#define IV_VALUE_LED_PERCENT12       ( 0x0C )    // Flag LedOn79,99%
#define IV_VALUE_LED_PERCENT13       ( 0x0D )    // Flag LedOn86,66%
#define IV_VALUE_LED_PERCENT14       ( 0x0E )    // Flag LedOn93,33%
#define IV_VALUE_LED_ON              ( 0x0F )    // Flag LedOn
// Bei setLED wird Param1 + Param2 + Param3 gesendet und als Antwort keiner erwartet.
// Bei getLED wird Param1 + Param2 gesendet und als Antwort Param1 + Param2 + Param3 erwartet.

// Active-Answer bei Abfrage von Information zu einem Modultyp
// Wenn in der Antwort auf GET_INFO als Information "Active-Answer" (case insensitiv) gesendet wird,
// dann wird im folgenden mittels Aktiver-Antwort auf die Information gewartet.
#define IV_FUNCTION_GET_INFO_OK                  0x004A

/*!
    /brief Lesen eines in seiner Größe unbestimmten Datenblocks.

    Der zu lesende Datenblock kann ein Wert einer Systemvariablen oder eine Datei sein. Die eigentlichen Daten werden
    mit ggf. mehreren read_data_block_ok befehlen gesendet. Das Ende des Datenblocks wird dadurch erkannt, dass ein 
    read_data_block_ok Befehl mit weniger Daten als in der Rückantwort durch buffer_size angegebenen Größe gelesen wird.
    Der übergebene timeout Wert entspricht dem auf "PC" Seite und wird bei der Antwort auf read_data_block_ok dazu verwendet um
    zum einen das Datenpaket ggf. erneut zu senden oder abzubrechen.

    \param[in] timeout Gibt den Timeout an, der bei der Active-Anwer für die Antwort verwendet wird.
    \param[in] buffer_size Gibt die maximal unterstützte Puffergröße der Active-Answer an. Ist nicht die maximale Paketgröße!
    \param[in] name Gibt eine Nullterminierte Zeichenfolge mit dem Namen der zu lesenden Systemvariablen oder wenn mit '/' beginnend, eine Datei an.

    \param[out] buffer_size Gibt die maximal unterstützte Puffergröße der Active-Answer an. Es werden so lange Datenpakete erwartet bis ein
                            Datenpaket mit kleinerem Dateninhalt gelesen wird.
*/
#define IV_FUNCTION_READ_DATA_BLOCK              0x004B
/*!
    \brief Active-Answer mit dem nächsten Dateninhalt zu der mit Read_data_block angeforderten Systemvariablen oder Datei.

    Software                        Firmware
            ----Read_data_block----->
            <--------Antwort---------

            <--Read_data_block_ok----
            ---------Antwort-------->

                    .........

            <--Read_data_block_ok----
            ---------Antwort-------->

    \param[in] buffer Datenjunk zu der angeforderten Systemvariablen oder Datei.
*/
#define IV_FUNCTION_READ_DATA_BLOCK_OK           0x004C

namespace Datafox
{
    namespace Communication
    {
        class ModuleSystem : public ModuleHardware
        {
        public:
            ModuleSystem( int32_t connectionNumber = ConnectionBase::defaultConnectionNumber(), uint8_t deviceAddress = ProtocolBase::defaultDeviceAddress() );
            virtual ~ModuleSystem();

        public:
            inline void setPasswordCommunication( const std::string &password )
            {
                m_passwordCommunication = password;
            }
            inline const std::string &passwordCommunication()
            {
                return m_passwordCommunication;
            }
            inline void setFlagPasswordProtected( int32_t state )
            {
                m_isPasswordProtected = (state == 0) ? 0 : 1;
            }
            // Ist Passwortschutz aktiv?
            inline int32_t isPasswordProtected()
            {
                return m_isPasswordProtected;
            }
            inline void setFlagPasswordProtection( int32_t state )
            {
                m_isPasswordProtection = (state == 0) ? 0 : 1;
            }
            // Wird Passwortschutz unterstützt?
            inline int32_t isPasswordProtection()
            {
                return m_isPasswordProtection;
            }

        private:
            std::string m_passwordCommunication; // Passwort fuer die Kommunikation
            int32_t m_isPasswordProtected; // Haelt die Information, dass das Geraet Passwortgeschuetzt ist.
            int32_t m_isPasswordProtection; // Haelt die Information, dass das Geraet den Passwortschutz unterstuetzt.

        public:
            std::string m_versionFirmware;

        public:
            int32_t writeData( const uint8_t *buffer, uint32_t writeCount, uint32_t &writeOutCount, uint8_t writeTrials = ConnectionBase::defaultWriteTrials() );
            int32_t readData( uint8_t *buffer, uint32_t readCount, uint32_t &readInCount, uint8_t readTrials = ConnectionBase::defaultReadTrials() );
            enum TraceDataType
            {
                tdt_write,
                tdt_read
            };
            enum TraceDataMode
            {
                tdm_full,
                tdm_fullWithCrlf,
                tdm_begin,
                tdm_append,
                tdm_end,
                tdm_endWithCrlf
            };
            void traceData( uint8_t *data, uint32_t size, TraceDataMode mode = tdm_fullWithCrlf, const char *prefix = nullptr );

        public:
            // Gibt Modulfehler zurueck.
            virtual void errorModule( uint16_t module, uint16_t error, std::string &text );
            // Gibt Funktionsfehlertext zurueck.
            virtual void errorFunction( uint16_t module, uint16_t func, uint16_t error, std::string &text );

            // MODULE_HARDWARE
            // BUG:
            // Im Bootloader 00.04.00.00 && 00.04.00.01 liefert die Funktion
            // - TypeTransponder den Wert fuer das Display
            // - TypeDisplay den Wert fuer das Keypad
            // - TypeKeypad den Wert fuer den Transponder
            int32_t writeDisplayType( uint8_t deviceAddress, uint8_t val, uint16_t value2 = 0xFFFF, bool checked = false );
            int32_t readDisplayType( uint8_t deviceAddress, uint8_t &val, bool checked = false );
            int32_t writeKeypadType( uint8_t deviceAddress, uint8_t val, uint16_t value2 = 0xFFFF, bool checked = false );
            int32_t readKeypadType( uint8_t deviceAddress, uint8_t &val, bool checked = false );
            int32_t writeTransponderReaderType( uint8_t deviceAddress, uint8_t val, uint16_t value2 = 0xFFFF, bool checked = false );
            int32_t readTransponderReaderType( uint8_t deviceAddress, uint8_t &val, bool checked = false );

            // Gesamte Hardwareinfo lesen
            int32_t readHardwareInfo( uint8_t deviceAddress, StructHardwareInfo &hardware_info );
            // Gesamte Hardwareinfo in String umsetzen.
            int32_t hardwareInfoToString( StructHardwareInfo hardwareInfo, std::string &info, std::list<std::pair<std::string, std::string>> *moduleList = nullptr );
    
            // MODULE_SYSTEM

            // Systemreset; Speicherbereiche loeschen
            int32_t systemReset( uint8_t deviceAddress, uint16_t modus );
            int32_t replySystemReset( uint8_t deviceAddress );

            // Firmwareversion lesen
            int32_t readFirmwareVersion( uint8_t deviceAddress, std::string &version );
            int32_t readBootloaderVersion( uint8_t deviceAddress, std::string &version );

            // Seriennummer lesen
            int32_t writeSerialNumber( uint8_t deviceAddress, uint32_t serial, uint32_t value2 = 0xFFFFFFFF );
            virtual int32_t readSerialNumber( uint8_t deviceAddress, uint32_t &serial );

            // System Device Log
            int32_t readSystemLogRecord( uint8_t deviceAddress, uint8_t *buffer, uint16_t &count, uint8_t &status );
            int32_t quitSystemLogRecord( uint8_t deviceAddress, uint8_t &status );
            int32_t restoreSystemLogRecords( uint8_t deviceAddress );

            // Datum/Uhrzeit setzen/lesen
            int32_t writeRtcDateTime( uint8_t deviceAddress, char* time );
            int32_t readRtcDateTime( uint8_t deviceAddress, char* time );

            // Helper für Systemvariablen von Mobile
            void latin1ToGsmCodec(std::string &buf );
            void gsmToLatin1Codec(std::string &buf);
            uint8_t hexToUint8( char* data );

            // Systemvariable setzen/lesen
            int32_t writeSystemVariable( uint8_t deviceAddress, const std::string &name, const std::string &val );
            int32_t readSystemVariable( uint8_t deviceAddress, const std::string &name, std::string &val );

            // Relais setzen/lesen
            int32_t writeRelayState( uint8_t deviceAddress, const std::string &name, uint8_t busNumber, uint8_t relayNumber, uint8_t status, uint16_t time );
            int32_t readRelayState( uint8_t deviceAddress, const std::string &name, uint8_t busNumber, uint8_t relayNumber, uint8_t &status, uint16_t &time );

        protected:
            uint32_t m_crcWriteSegment;
            uint32_t m_crcReadSegment;
            std::string m_streamBuffer;
            bool m_streamTransfer;
        public:
            void setStreamTransfer( bool state )
            { 
                m_streamTransfer = state;
            }
            bool isStreamTransfer()
            { 
                return m_streamTransfer;
            }
            void clearStreamTransfer()
            { 
                m_streamBuffer.clear();
            }

        protected:
            std::mutex m_mutexSegment;      // Segmentuebertragung

        public:
            std::set<uint8_t> controllerSegmentKeys()
            {
                std::set<uint8_t> keys;
                for ( auto iter = m_mapControllerSegment.cbegin(); iter != m_mapControllerSegment.cend(); ++iter )
                {
                    keys.insert(iter->first);
                }

                return keys;
            }

            inline int32_t controllerSegmentActiveCount( ControllerSegment::BlockTransferType blockType = ControllerSegment::btt_mask_known )
            {
                int32_t counter = 0;
                std::set<uint8_t> keys( controllerSegmentKeys() );
                for ( auto iter = keys.cbegin(); iter != keys.cend(); ++iter )
                {
                    // Globale-Object uebergehen.
                    if ( *iter == 255 )
                    {
                        continue;
                    }

                    if ( (blockType == ControllerSegment::btt_mask_known || blockType == m_mapControllerSegment[*iter].transferType) 
                        && m_mapControllerSegment[*iter].transferState == ControllerSegment::BlockTransferState::active )
                    {
                        ++counter;
                    }
                }

                return counter;
            }

            // Setzt in allen ControllSegment die ggf. vom Global-Objekt uebernommene duration auf 0.
            inline void resetControllerSegmentGlobalDuration()
            {
                std::set<uint8_t> keys( controllerSegmentKeys() );
                for ( auto iter = keys.cbegin(); iter != keys.cend(); ++iter )
                {
                    // Globale-Object uebergehen.
                    if ( *iter == 255 )
                    {
                        continue;
                    }

                    if ( m_mapControllerSegment[*iter].transferFlags.settingsFromGlobal )
                    {
                        m_mapControllerSegment[*iter].allowedTransferTypes = ControllerSegment::btt_none;
                        m_mapControllerSegment[*iter].duration = std::chrono::seconds(0);
                        m_mapControllerSegment[*iter].transferFlags.settingsFromGlobal = false;
                    }
                }
            }

            inline ControllerSegment &controllerSegment( uint8_t deviceAddress )
            {
                return m_mapControllerSegment[deviceAddress];
            }

        protected:
            boost::ptr_map<uint8_t, ControllerSegment> m_mapControllerSegment;

        public:
            // Datensegmentuebertragung beginnen/enden
            int32_t enterWriteDataSegment( uint8_t deviceAddress, uint16_t art, uint32_t &address, uint32_t size, uint32_t crc );
            int32_t replyEnterWriteDataSegment( uint8_t deviceAddress );
            int32_t exitWriteDataSegment( uint8_t deviceAddress, uint16_t art, uint32_t crc );
            int32_t replyExitWriteDataSegment( uint8_t deviceAddress );

            // Datensegmentuebertragung beginnen/enden
            int32_t enterReadDataSegment( uint8_t deviceAddress, uint16_t art, uint32_t &address, uint32_t &size, uint32_t &crc );
            int32_t replyEnterReadDataSegment( uint8_t deviceAddress );
            int32_t exitReadDataSegment( uint8_t deviceAddress, uint16_t art );
            int32_t replyExitReadDataSegment( uint8_t deviceAddress );

            // Flash setzen/lesen mittels logischer Adresse (die ersten 16kByte Bootloader koennen nur gelesen werden)
            int32_t writeToFlash( uint8_t deviceAddress, uint32_t address, uint8_t *buffer, uint32_t count );
            int32_t readFromFlash( uint8_t deviceAddress, uint32_t address, uint8_t *buffer, uint32_t count );
            int32_t readFlashPage( uint8_t deviceAddress, uint32_t pageNumber, uint32_t offset, uint8_t *buffer, uint32_t &count );
            // Flashdaten lesen, mittels physikalischer Adresse.
            int32_t readFromFlashPhysical( uint8_t deviceAddress, uint32_t address, uint8_t *buffer, uint32_t count );

            // EEPROM setzen/lesen
            int32_t writeToEeprom( uint8_t deviceAddress, uint32_t address, uint8_t *buffer, uint32_t count );
            int32_t readFromEeprom( uint8_t deviceAddress, uint32_t address, uint8_t *buffer, uint32_t count );

            // PROGRAM setzen/(lesen unterbunden)
            int32_t writeToProgramFlash( uint8_t deviceAddress, uint32_t address, uint8_t *buffer, uint32_t count );
            int32_t readFromProgramFlash( uint8_t deviceAddress, uint32_t address, uint8_t *buffer, uint32_t count );

            // RAM setzen/lesen
            int32_t writeToRam( uint8_t deviceAddress, uint32_t address, uint8_t *buffer, uint32_t count );
            int32_t readFromRam( uint8_t deviceAddress, uint32_t address, uint8_t *buffer, uint32_t count );

            // MMC schreiben/lesen
            int32_t writeToMmc( uint8_t deviceAddress, uint32_t address, uint8_t *buffer, uint32_t count );
            int32_t readFromMmc( uint8_t deviceAddress, uint32_t address, uint8_t *buffer, uint32_t count );

            // Info zum Dateisystem
            int32_t readSfsMpb( uint8_t deviceAddress, uint8_t &type, SFSMPB &sfs );

            // Systembios aufrufen/verlassen
            int32_t enterBiosMode( uint8_t deviceAddress );
            int32_t replyEnterBiosMode( uint8_t deviceAddress );
            int32_t exitBiosMode( uint8_t deviceAddress );
            int32_t replyExitBiosMode( uint8_t deviceAddress );

            // Interface setzen/lesen (RS232, TCPIP...)
            int32_t writeMainInterfaceType( uint8_t deviceAddress, uint8_t type );
            int32_t readMainInterfaceType( uint8_t deviceAddress, uint8_t &type );
    
            // Baud-Rate setzen/lesen
            int32_t writeBaud( uint8_t deviceAddress, uint32_t baud );
            int32_t readBaud( uint8_t deviceAddress, uint32_t &baud );

            // DeviceID setzen/lesen
            int32_t writeDeviceNumber( uint8_t deviceAddress, uint16_t address );
            int32_t readDeviceNumber( uint8_t deviceAddress, uint16_t &address );

            // Bootloader aktivieren
            int32_t enterBootloaderMode( uint8_t deviceAddress);

            // Monitor Mode vom XPort
            int32_t enterXportMonitorMode( uint8_t deviceAddress, char* data, uint16_t &count );
            int32_t replyEnterXportMonitorMode( uint8_t deviceAddress );
            int32_t enterXportMonitorModeExt( uint8_t deviceAddress, char* data, uint16_t &count );
            int32_t replyEnterXportMonitorModeExt( uint8_t deviceAddress );
            int32_t xportMonitorModeCommand( uint8_t deviceAddress, char* buffer, uint16_t &count );
            int32_t exitXportMonitorMode( uint8_t deviceAddress );
            int32_t replyExitXportMonitorMode( uint8_t deviceAddress );

            // Transparentsmodus fuer COM
            int32_t enterComServerMode( uint8_t deviceAddress, const char* comm, uint32_t baud, uint8_t byteSize, uint8_t parity, uint8_t stopBits, uint16_t flags, uint16_t timeoutToClose );
            int32_t replyEnterComServerMode( uint8_t deviceAddress );
            int32_t exitComServerMode( uint8_t deviceAddress );

            // Informationen zu einem Modultyp abfragen
            int32_t readModuleInfo(uint8_t deviceAddress, const std::string &name, const std::string &param, std::string &info);
            int32_t replyReadModuleInfo( uint8_t deviceAddress );

            // Aktivieren/Deaktivieren und Abfragen der Benachrichtigungen
            int32_t writeNotify( uint8_t deviceAddress, uint8_t type, uint8_t state );
            int32_t readNotify( uint8_t deviceAddress, uint8_t type, uint8_t &state );

            // Softwareinformation der Anwendungsfirmware abfragen
            int32_t readSoftwareInfo( uint8_t deviceAddress, uint8_t* info, int32_t &infoLength );

            // Setzen/Lesen der Systemflags
            int32_t writeSystemFlags( uint8_t deviceAddress, uint32_t flags );
            int32_t readSystemFlags( uint8_t deviceAddress, uint32_t &flags );
    
            // Passwort pruefen
            int32_t writeSetupPassword( uint8_t deviceAddress, const std::string &password );
            // Passwortschlüssel abrufen
            int32_t readSetupPasswordKey( uint8_t deviceAddress, char* key );

            // Einen Tastencode ausführen. F1-F15 drücken.
            int32_t writeVirtualKey( uint8_t deviceAddress, uint8_t key_code, uint8_t flags );

            // Daten als ein Datenstrom senden.
            int32_t writeStream( uint8_t deviceAddress, Stream_type type, const char *buffer, uint32_t count );
            // Daten als ein Datenstrom empfangen.
            int32_t readStream( uint8_t deviceAddress, Stream_type type, char *buffer, uint32_t count );

            // Systemschlüssel pruefen
            int32_t writeSystemKey( uint8_t deviceAddress, char* key );
            // Systemschlüssel abrufen
            int32_t readSystemKey( uint8_t deviceAddress, char* key );

        private:
            enum class TranparencyMode
            {
                off,    // Nicht aktiv.
                masked, // Aktiv, wobei Sonderzeichen maskiert werden.
                raw     // Aktiv, alles wird 1:1 weitergeleitet.
            };
            TranparencyMode m_transparencyMode;
        public:
            // Daten transparent übertragen.
            int32_t enterTransparencyMode( uint8_t deviceAddress, uint16_t id, uint16_t flags, uint32_t readBufferSize, uint32_t writeBufferSize, uint32_t timeout );
            int32_t exitTransparencyMode( uint8_t deviceAddress );
            bool isTransparencyMode();

            // Gelbe LED bei Dockingstation-V2 setzen/lesen
            int32_t writeLedState( uint8_t deviceAddress, uint8_t slot, uint8_t id, uint8_t state );
            int32_t readLedState( uint8_t deviceAddress, uint8_t slot, uint8_t id, uint8_t &state );

            // Datenblock lesen
            int32_t readDataBlock( uint8_t deviceAddress, const std::string &name, std::vector<uint8_t> &data );
            int32_t replyReadDataBlock( uint8_t deviceAddress, uint8_t *data, uint16_t size );

        private:
            std::map<uint8_t, std::vector<uint8_t>> m_mapDataBlock;

        public:
            // Ausfuehren eines Protokollbefehls
            virtual int32_t call( uint16_t context, uint16_t module, uint16_t function, uint8_t *buffer, uint16_t &count, int32_t contextValue = -1 );

            //#################################
            // remoteCall - Routinen

            // Ausfuehren von Befehlen
            virtual int32_t remoteCall( uint16_t module, uint16_t func, uint8_t *data, uint16_t &count );

        #ifdef WITH_VIRTUAL_DEVICE
        protected:
            type_variable m_variables;
            std::vector<Segment_data> m_segments;
        #endif
        };

    } // namespace Communication
} // namespace Datafox

#endif
